<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\SeedDonation;
use App\Models\SeedRequest;
use App\Models\Seeds;
use App\User;
use Response;

class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    /*public function __construct()
    {
        $this->middleware('auth');
    }*/

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        return view('home');
    }

    public function seedsAPIList(Request $request) {
        $seeds = Seeds::where('common_name', '!=', '');
        if($request->has('searchString') && $request->get('searchString') != '') {
            $seeds->whereRaw("common_name like '%".$request->get('searchString')."%' or sun_exposure like '%".$request->get('searchString')."%' or bloom_colour like '%".$request->get('searchString')."%' or harvesting_location like '%".$request->get('searchString')."%'");
        }
        if($request->has('filter1') && $request->get('filter1') != '') {
            $seeds->where('seed_family', $request->get('filter1'));
        }
        if($request->has('filter2') && $request->get('filter2') != '') {
            $seeds->whereIn('sun_exposure', $request->get('filter2'));
        }
        if($request->has('filter3') && $request->get('filter3') != '') {
            $seeds->whereIn('soil_moisture', $request->get('filter3'));
        }
        if($request->has('filter4') && $request->get('filter4') != '') {
            $seeds->whereIn('bloom_colour', $request->get('filter4'));
        }
        if($request->has('order') && $request->get('order') != '') {
            $seeds->orderBy($request->get('order'), 'asc');
        }
        return Response::json($seeds->get());
    }

    public function trees() {
        $type = 'Trees';
        return view('products', compact('type'));
    }

    public function shrubs() {
        $type = 'Shrubs';
        return view('products', compact('type'));
    }

    public function herbs() {
        $type = 'Herbs';
        return view('products', compact('type'));
    }

    public function forbs() {
        $type = 'Forbs';
        return view('products', compact('type'));
    }

    public function others() {
        $type = 'Others';
        return view('products', compact('type'));
    }

    public function aboutus() {
        return view('aboutus');
    }

    public function seedDonation() {
        return view('seed-donation');
    }

    public function seedRequest() {
        return view('seed-request');
    }

    public function seedRequests() {
        $requests = SeedRequest::leftJoin('seeds', 'seeds.id', 'seed_request.seed_id')->selectRaw('seeds.common_name, seed_request.*')->orderBy('seed_request.id', 'desc')->get();
        return view('requests', compact('requests'));
    }

    public function seedDonations() {
        $donations = SeedDonation::leftJoin('seeds', 'seeds.id', 'seed_donations.seed_id')->selectRaw('seeds.common_name, seed_donations.*')->orderBy('seed_donations.id', 'desc')->get();
        return view('donations', compact('donations'));
    }
    
    public function otherSeedDonations() {
        $seed = Seeds::pluck('common_name');
        $donations = SeedDonation::leftJoin('seeds', 'seeds.id', 'seed_donations.seed_id')->selectRaw('seeds.common_name, seed_donations.*')->whereNotIn('seed_donations.seed_id', $seed)->orderBy('seed_donations.id', 'desc')->get();
        return view('otherDonations', compact('donations'));
    }

    public function usersList() {
        $users = User::get();
        return view('users', compact('users'));
    }

    public function usersEdit($id) {
        $user = User::findOrFail($id);
        return view('users-edit', compact('user'));
    }

    public function usersDelete($id) {
        $user = User::findOrFail($id);
        $user->delete();
        return redirect()->route('users.list');
    }

    public function usersUpdate($id, Request $request) {
        $data = $request->all();
        $user = User::findOrFail($id);
        $user->update($data);
        return redirect()->route('users.list');
    }

    public function seedsList() {
        $seeds = Seeds::orderBy('id', 'desc')->get();
        return view('seeds-list',compact('seeds'));
    }

    public function seedDetails($id) {
        $seed = Seeds::findOrFail($id);
        return view('seed-details',compact('seed'));
    }

    public function createSeed(Request $request) {
        $donation = [];
        if($request->has('donation_id')) {
            $donation = SeedDonation::findOrFail($request->get('donation_id'));
        }
        return view('seed-create', compact('donation'));
    }

    public function editSeed($id) {
        $seed = Seeds::findOrFail($id);
        return view('seed-edit', compact('seed'));
    }

    public function deleteSeed($id) {
        $seed = Seeds::findOrFail($id);
        $seed->delete();
        return redirect()->route('seeds.list');
    }

    public function seedStore(Request $request) {
        $requestData = $request->all();
        $rules = array(
            'seed_id' => 'required|unique:seeds',
            'common_name' => 'required|unique:seeds'
        );

        $validator = \Validator::make($requestData, $rules);
        if($request->hasFile('seed_file')){
            $imageName = time().'.'.$request->seed_file->extension(); 
            $requestData['seed_file'] = $imageName;
        }
        $seed = new Seeds();
        if($validator->passes()) {
            $seed->create($requestData);
            if($request->hasFile('seed_file')){
                $request->seed_file->move(public_path('assets/images/seeds'), $imageName);
            }
            $seeds = Seeds::orderBy('id', 'desc')->get();
            return view('seeds-list',compact('seeds'));
        } else {
            return redirect()->back()->withInput()->with('error', 'seed already exist with same seed id');
        }
    }

    public function seedUpdate($id, Request $request) {
        $requestData = $request->all();
        if($request->hasFile('seed_file')){
            $request->validate([
                'seed_file' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
            ]);
            $imageName = time().'.'.$request->seed_file->extension(); 
            $requestData['seed_file'] = $imageName;
        }
        $seed = Seeds::findOrFail($id);
        $seed->update($requestData);
        if($request->hasFile('seed_file')){
            $request->seed_file->move(public_path('assets/images/seeds'), $imageName);
        }
        $seeds = Seeds::orderBy('id', 'desc')->get();
        return view('seeds-list',compact('seeds'));
    }

    public function contactus() {
        return view('contactus');
    }

    public function contactSendMail(Request $request) {
        try {
            \Mail::to('jason.kerr@flemingcollege.ca')->send(new \App\Mail\sendContactUs($request->all()));
        } catch(\Exception $e){}
        $arrayName = array('type' => 'success', 'msg' => "Request Submitted Successfully.");
        return Response::json($arrayName);
    }

    public function terms() {
        return view('terms-and-conditions');
    }

    public function storeSeedDonation(Request $request) {
        $seedDonation = new SeedDonation();
        $data = $request->all();
        $seedDonation->create($data);
        $seeds = Seeds::orderBy('id', 'desc')->get();
        try {
            $emails = array('jason.kerr@flemingcollege.ca', auth()->user()->email);
            \Mail::to($emails)->send(new \App\Mail\sendSeedDonation($data));
            //\Mail::to(auth()->user()->email)->send(new \App\Mail\sendSeedDonation($data));
        } catch(\Exception $e){}
        $p = 1;
        return view('seed-donation', compact('p', 'seeds'));
    }

    public function storeSeedRequest(Request $request) {
        $seed = Seeds::where('common_name', $request->seed_name)->first();
        $seeds = Seeds::orderBy('id', 'desc')->get();
        if($seed) {
            $weight = $seed->weight;
            if($request->quantity > $weight) {
                $p=2;
                return view('seed-request', compact('p', 'seeds'));
            }
        }
        $seedRequest = new SeedRequest();
        $data = $request->all();
        $seedRequest->create($data);
        try {
            $emails = array('jason.kerr@flemingcollege.ca', auth()->user()->email);
            \Mail::to($emails)->send(new \App\Mail\sendSeedRequest($data));
            //\Mail::to()->send(new \App\Mail\sendSeedRequest($data));
        } catch(\Exception $e){ \Log::info($e); }
        $p = 1;
        
        return view('seed-request', compact('p', 'seeds'));
    }

    public function updateSeedRequest($id, Request $request) {
        $seedRequest = SeedRequest::findOrFail($id);
        $data = $request->all();
        $user = User::findOrFail($seedRequest->user_id);
        $pervStat = $seedRequest->status;
        $seedRequest->update($data);
        if($data['status'] == 'Completed') {
            $quantity = $data['acceptable_amount'];
            $seed = Seeds::where('common_name', $seedRequest->seed_name)->first();
            if($seed) {
                $weight = $seed->weight;
                \Log::info($weight." ".$quantity);
                $weight = $weight - $quantity;
                $seed->weight = $weight;
                $seed->save();
            }
        }
        if($pervStat == 'Completed' && $data['status'] == 'Cancellation') {
            $quantity = $seedRequest->acceptable_amount;
            $seed = Seeds::where('common_name', $seedRequest->seed_name)->first();
            if($seed) {
                $weight = $seed->weight;
                $weight = $weight + $quantity;
                $seed->weight = $weight;
                $seed->save();
            }
        }
        if($pervStat != $data['status']) {
            try{
                $data['username'] = $user->first_name." ".$user->last_name;
                \Mail::to($user->email)->send(new \App\Mail\RequestStatusChange($data));
            } catch(\Exception $e){ \Log::info($e); }
        }
        return redirect()->route('seed.requests');
    }

    public function editSeedRequest($id) {
        $seedRequest = SeedRequest::findOrFail($id);
        return view('requests-edit', compact('seedRequest'));
    }

    public function deleteSeedRequest($id) {
        $seedRequest = SeedRequest::findOrFail($id);
        $seedRequest->delete();
        return redirect()->route('seed.requests');
    }

    public function updateSeedDonation($id, Request $request) {
        $seedDonation = SeedDonation::findOrFail($id);
        $data = $request->all();
        $user = User::findOrFail($seedDonation->user_id);
        $pervStat = $seedDonation->status;
        $seedDonation->update($data);
        if($data['status'] == 'Completed') {
            $quantity = $seedDonation->acceptable_amount;
            $seed = Seeds::where('common_name', $seedDonation->seed_id)->first();
            if($seed) {
                $weight = $seed->weight;
                $weight = $weight + $quantity;
                $seed->weight = $weight;
                $seed->save();
            }
        }
        if($pervStat != $data['status']) {
            try {
                $data['username'] = $user->first_name." ".$user->last_name;
                \Mail::to($user->email)->send(new \App\Mail\DonationStatusChange($data));
            } catch(\Exception $e){ \Log::info($e); }
        }
        return redirect()->route('seed.donations');
    }

    public function editSeedDonation($id) {
        $seedDonation = SeedDonation::findOrFail($id);
        return view('donation-edit', compact('seedDonation'));
    }

    public function deleteSeedDonation($id) {
        $seedRequest = SeedDonation::findOrFail($id);
        $seedRequest->delete();
        return redirect()->route('seed.donations');
    }

    public function getSeeds(Request $request) {
        $seeds = Seeds::where('seed_family', $request->seed_family)->orderBy('id', 'desc')->get();
        return Response::json($seeds);
    }

    public function seedRequestView($id) {
        $seed = Seeds::findOrFail($id);
        $seedID = $seed->id;
        return view('seed-request', compact('seedID'));
    }

    public function seedDonationView($id) {
        $seed = Seeds::findOrFail($id);
        $seedID = $seed->id;
        return view('seed-donation', compact('seedID'));
    }
}
